// Import necessary D3 functions for data visualization
import { select, selectAll, scaleOrdinal } from 'd3';

// Select the SVG container element and define width and height variables
const svg = select('svg');
const width = +svg.attr('width'); 
const height = +svg.attr('height');
const centre_x = width / 2; // Calculate the center X-coordinate
const centre_y = height / 2; // Calculate the center Y-coordinate
const colorScale = ["#c980f6"]; // Define the color scale for nodes

// Set up responsive design for the SVG container
svg.attr("preserveAspectRatio", "xMidYMid meet")
   .attr("viewBox", [0, 0, width * 2, height * 2]);

// Define marker for links in the graph
const defs = svg.append("defs");
defs.selectAll("marker")
    .data(["node"])
    .enter()
    .append("marker")
        .attr("id", d => d)
        .attr("viewBox", "0 -3.7 7 7")
        .attr("refX", 16)
        .attr("refY", 0)
        .attr("markerWidth", 5)
        .attr("markerHeight", 5)
        .attr("orient", "auto")
    .append("path")
        .attr("d", "M0,-2.5L5,0L0,2.5")
        .attr("fill", "#999");

// Load data from 'data.json' and process it
d3.json('data.json').then(data => {
  
  // Extract and define links from the data
  const links = data.links.map(d => Object.create(d));
  links.forEach((d, i) => {
    d.srcType = data.links[i].srcType;
    d.tgtType = data.links[i].tgtType;
    d.relationship = data.links[i].relationship;
    d.value = data.links[i].value;
  });
  
  // Extract and define nodes from the data
  const nodes = data.nodes.map(d => Object.create(d));
  nodes.forEach((d, i) => {
    d.group = data.nodes[i].group;
  });

  // Generate the network layout using D3's force simulation
  const simulation = d3.forceSimulation(nodes)
      .force("link", d3.forceLink(links)
             .id(d => d.id)
             .distance(120))
      .force("charge", d3.forceManyBody()
             .strength(-500)
            )
      .force("collision", d3.forceCollide()
                 .radius(35)
            )
      .force("center", d3.forceCenter(width, height));

  // Define the drag behavior for nodes
  const drag = simulation => {
    function dragstarted(event) {
      if (!event.active) simulation.alphaTarget(0.5).restart();
      event.subject.fx = event.subject.x;
      event.subject.fy = event.subject.y;
    }
    function dragged(event) {
      event.subject.fx = event.x;
      event.subject.fy = event.y;
    }
    function dragended(event, d) {
      if (!event.active) simulation.alphaTarget(0);
      event.subject.fx = null;
      event.subject.fy = null;
    }
    return d3.drag()
      .on("start", dragstarted)
      .on("drag", dragged)
      .on("end", dragended);
  }

  // Append links to the SVG using marker for direction
  const link = svg.append("g")
      .selectAll("line")
      .data(links)
      .join("line")
      .attr("marker-end", "url(#node)");

  // Append nodes to the SVG and enable dragging behavior
  const node = svg.append("g")
      .attr("class", "node")
      .selectAll("g")
      .data(nodes)
      .enter()
      .append("g")
      .attr("transform", d => "translate(" + centre_x + ", " + centre_y + ")");
  
  // Append circles representing nodes
  const circles = node.append("circle")
      .attr("r", 35)
      .attr("fill", d => colorScale[0])
      .on("mouseover", focusNode) // Event listeners for node mouseover and mouseout
      .on("mouseout", blurNode)
      .call(drag(simulation));

  // Append text labels for nodes
  const nodeLbl = node.append("text")
      .text(d => d.id)
      .attr("dy", "0.3em");
  
  // Define the tick function for the simulation (positions of nodes and links)
  simulation.on("tick", () => {
    link
      .attr("x1", d => d.source.x)
      .attr("y1", d => d.source.y)
      .attr("x2", d => d.target.x)
      .attr("y2", d => d.target.y);
    node
      .attr("transform", d => "translate(" + d.x + ", " + d.y + ")");
  });
  
  // Event handler for node mouseover (highlight node)
  function focusNode() {
    d3.select(this).classed("nodeHover", true);
  }

  // Event handler for node mouseout (remove node highlight)
  function blurNode() {
    d3.select(this).classed("nodeHover", false);
  }

});
